/*----------------------------------------------------------------------
|       includes
+---------------------------------------------------------------------*/
#ifndef STRICT
#define STRICT
#endif

#include "stdafx.h"

#include "registry.h"
#include "utils.h"

/*----------------------------------------------------------------------
| OpenPocketTVKey
+---------------------------------------------------------------------*/
LONG
OpenPocketTVKey(HKEY *pKey)
{
	return RegOpenKeyEx(HKEY_CURRENT_USER, TEXT("Software\\MpegTV\\PocketTV"), 0, 0, pKey);
}

/*----------------------------------------------------------------------
| SetRegistryValue
+---------------------------------------------------------------------*/
LONG
SetRegistryValue(const TCHAR *pValue, DWORD type, const BYTE *ptr, DWORD size)
{
	HKEY key;
	LONG ret;

	ret = OpenPocketTVKey(&key);
	if (ret != ERROR_SUCCESS) {
		return ret;
	}
	ret = RegSetValueEx(key, pValue, 0, type, ptr, size);
	RegCloseKey(key);
	return ret;
}

/*----------------------------------------------------------------------
| DeleteRegistryValue
+---------------------------------------------------------------------*/
LONG
DeleteRegistryValue(const TCHAR *pValue)
{
	HKEY key;
	LONG ret;

	ret = OpenPocketTVKey(&key);
	if (ret != ERROR_SUCCESS) {
		return ret;
	}
	ret = RegDeleteValue(key, pValue);
	RegCloseKey(key);
	return ret;
}

/*----------------------------------------------------------------------
| GetRegistryValueFromKey
+---------------------------------------------------------------------*/
LONG
GetRegistryValueFromKey(HKEY hKey, const TCHAR *pSubKey, const TCHAR *pValue, DWORD type, BYTE *ptr, DWORD *pSize)
{
	HKEY key;
	LONG ret;
	DWORD actual_type;

	ret = RegOpenKeyEx(hKey, pSubKey, 0, 0, &key);

	if (ret != ERROR_SUCCESS) {
		return ret;
	}
	ret = RegQueryValueEx(key, pValue, 0, (LPDWORD)&actual_type, ptr, pSize);
	RegCloseKey(key);
	if (ret == ERROR_SUCCESS) {
		if (type == actual_type) {
			return ERROR_SUCCESS;
		} else {
			return -1;
		}
	}
	return ret;
}

/*----------------------------------------------------------------------
| GetRegistryValue
+---------------------------------------------------------------------*/
LONG
GetRegistryValue(const TCHAR *pValue, DWORD type, BYTE *ptr, DWORD *pSize)
{
	return GetRegistryValueFromKey(HKEY_CURRENT_USER, TEXT("Software\\MpegTV\\PocketTV"), pValue, type, ptr, pSize);
}

/*----------------------------------------------------------------------
| SetRegistryValueDWORD
+---------------------------------------------------------------------*/
LONG
SetRegistryValueDWORD(const TCHAR *pValue, DWORD value)
{
	return SetRegistryValue(pValue, REG_DWORD, (const BYTE *)&value, sizeof value);
}

/*----------------------------------------------------------------------
| GetRegistryValueDWORD
+---------------------------------------------------------------------*/
LONG
GetRegistryValueDWORD(const TCHAR *pValue, DWORD *p)
{
	DWORD size = sizeof(DWORD);

	return GetRegistryValue(pValue, REG_DWORD, (BYTE *)p, &size);
}

/*----------------------------------------------------------------------
| GetRegistryValueFromKeyDWORD
+---------------------------------------------------------------------*/
LONG
GetRegistryValueFromKeyDWORD(HKEY hKey, const TCHAR *pSubKey, const TCHAR *pValue, DWORD *p)
{
	DWORD size = sizeof(DWORD);

	return GetRegistryValueFromKey(hKey, pSubKey, pValue, REG_DWORD, (BYTE *)p, &size);
}

/*----------------------------------------------------------------------
| GetRegistryValueFromKeyAny
+---------------------------------------------------------------------*/
LONG
GetRegistryValueFromKeyAny(HKEY hKey, const TCHAR *pSubKey, const TCHAR *pValue, DWORD type, BYTE **ptr, DWORD *pSize)
{
	BYTE *p = NULL;
//	TCHAR buf[1024];

	HKEY key = NULL;
	LONG ret;
	DWORD actual_type;
	DWORD size;

	ret = RegOpenKeyEx(hKey, pSubKey, 0, 0, &key);
	if (ret != ERROR_SUCCESS) {
		return ret;
	}
	if (pSize == NULL) {
		pSize = &size;
	}
	*pSize = (DWORD)-1;
	ret = RegQueryValueEx(key, pValue, 0, (LPDWORD)&actual_type, NULL, pSize);
//	size = sizeof(buf);
//	ret = RegQueryValueEx(key, TEXT("Loop"), 0, (LPDWORD)&actual_type, (BYTE *)buf, &size);
	if (ret != ERROR_SUCCESS) {
		goto err;
	}
	if (actual_type != type) {
		ret = -1;
		goto err;
	}
	if (ptr != NULL) {
		p = (BYTE *)pockettv_malloc(*pSize);
		if (p == NULL) {
			ret = -1;
			goto err;
		}
		ret = RegQueryValueEx(key, pValue, 0, (LPDWORD)&actual_type, p, pSize);
		if (ret != ERROR_SUCCESS) {
			goto err;
		}
		*ptr = p;
	}
	RegCloseKey(key);
	return ERROR_SUCCESS;

err:
	if (key != NULL) {
		RegCloseKey(key);
	}
	if (p != NULL) {
		free((void *)p);
	}
	return ret;
}

/*----------------------------------------------------------------------
| GetRegistryValueAny
+---------------------------------------------------------------------*/
LONG
GetRegistryValueAny(const TCHAR *pValue, DWORD type, BYTE **ptr, DWORD *pSize)
{
	return GetRegistryValueFromKeyAny(HKEY_CURRENT_USER, TEXT("Software\\MpegTV\\PocketTV"), pValue, type, ptr, pSize);
}

#if 0
/*----------------------------------------------------------------------
| GetRegistryValueAny
+---------------------------------------------------------------------*/
LONG
GetRegistryValueAny(const TCHAR *pValue, DWORD type, BYTE **ptr, DWORD *pSize)
{
	BYTE *p = NULL;
//	TCHAR buf[1024];

	HKEY key = NULL;
	LONG ret;
	DWORD actual_type;
	DWORD size;

	ret = OpenPocketTVKey(&key);
	if (ret != ERROR_SUCCESS) {
		return ret;
	}
	if (pSize == NULL) {
		pSize = &size;
	}
	*pSize = (DWORD)-1;
	ret = RegQueryValueEx(key, pValue, 0, (LPDWORD)&actual_type, NULL, pSize);
//	size = sizeof(buf);
//	ret = RegQueryValueEx(key, TEXT("Loop"), 0, (LPDWORD)&actual_type, (BYTE *)buf, &size);
	if (ret != ERROR_SUCCESS) {
		goto err;
	}
	if (actual_type != type) {
		ret = -1;
		goto err;
	}
	if (ptr != NULL) {
		p = (BYTE *)pockettv_malloc(*pSize);
		if (p == NULL) {
			ret = -1;
			goto err;
		}
		ret = RegQueryValueEx(key, pValue, 0, (LPDWORD)&actual_type, p, pSize);
		if (ret != ERROR_SUCCESS) {
			goto err;
		}
		*ptr = p;
	}
	RegCloseKey(key);
	return ERROR_SUCCESS;

err:
	if (key != NULL) {
		RegCloseKey(key);
	}
	if (p != NULL) {
		free((void *)p);
	}
	return ret;
}
#endif // 0

// eof